@@ -73,6 +73,9 @@ extern idCVar		com_showAsyncStats;
 extern idCVar		com_showSoundDecoders;
 extern idCVar		com_makingBuild;
 extern idCVar		com_updateLoadSize;
+extern idCVar		com_enableDebuggerServer;
+extern idCVar		com_dbgClientAdr;
+extern idCVar		com_dbgServerAdr;
 
 extern int			time_gameFrame;			// game logic time
 extern int			time_gameDraw;			// game present time
@@ -84,6 +87,8 @@ extern volatile int	com_ticNumber;			// 60 hz tics, incremented by async functio
 extern int			com_editors;			// current active editor(s)
 extern bool			com_editorActive;		// true if an editor has focus
 
+extern bool			com_debuggerSupported;	// only set to true when the updateDebugger function is set. see GetAdditionalFunction()
+
 #ifdef _WIN32
 const char			DMAP_MSGID[] = "DMAPOutput";
 const char			DMAP_DONE[] = "DMAPDone";
@@ -111,6 +116,8 @@ struct MemInfo_t {
 };
 
 class idLangDict;
+class idInterpreter;
+class idProgram;
 
 class idCommon {
 public:
@@ -158,6 +165,7 @@ public:
 								// Writes cvars with the given flags to a file.
 	virtual void				WriteFlaggedCVarsToFile( const char *filename, int flags, const char *setCmd ) = 0;
 
+
 								// Begins redirection of console output to the given buffer.
 	virtual void				BeginRedirect( char *buffer, int buffersize, void (*flush)( const char * ) ) = 0;
 
@@ -265,6 +273,11 @@ public:
 		// it returns true if we're currently running the doom3 demo
 		// not relevant for mods, only for game/ aka base.dll/base.so/...
 		FT_IsDemo = 1,
+		// the function's signature is bool fn(idInterpreter,idProgram,int) with arguments:
+		// idInterpreter *interpreter, idProgram *program, int instructionPointer
+		// it returns true if the game debugger is active.
+		// relevant for mods.
+		FT_UpdateDebugger,
 	};
 
 	// returns true if that function is available in this version of dhewm3
