@@ -132,6 +132,25 @@ public:
 	void			Swap( idList<type> &other );						// swap the contents of the lists
 	void			DeleteContents( bool clear );						// delete the contents of the list
 
+#if __cplusplus >= 201103L || defined(_MSVC_LANG) && _MSVC_LANG >= 201103L
+	// begin and end allow using C++11 for(ElemType& e : myIdlist) { ... }
+	type* begin() {
+		return list;
+	}
+
+	const type* begin() const {
+		return list;
+	}
+
+	type* end() {
+		return list + num;
+	}
+
+	const type* end() const {
+		return list + num;
+	}
+#endif
+
 private:
 	int				num;
 	int				size;
