@@ -27,8 +27,6 @@ If you have questions concerning this license or the applicable additional terms
 */
 
 // DG: replace libjpeg with stb_image.h because it causes fewer headaches
-// include this first, otherwise build breaks because of  use_idStr_* #defines in Str.h
-#define STB_IMAGE_IMPLEMENTATION
 #define STBI_NO_HDR
 #define STBI_NO_LINEAR
 #define STBI_ONLY_JPEG // at least for now, only use it for JPEG
