@@ -26,6 +26,7 @@ If you have questions concerning this license or the applicable additional terms
 ===========================================================================
 */
 #include "sys/platform.h"
+#include "sys/sys_imgui.h"
 
 #include "renderer/tr_local.h"
 
@@ -531,6 +532,8 @@ const void	RB_SwapBuffers( const void *data ) {
 		RB_ShowImages();
 	}
 
+	D3::ImGuiHooks::EndFrame();
+
 	int fillAlpha = r_fillWindowAlphaChan.GetInteger();
 	if ( fillAlpha == 1 || (fillAlpha == -1 && glConfig.shouldFillWindowAlpha) )
 	{
