@@ -35,6 +35,9 @@ If you have questions concerning this license or the applicable additional terms
 
 #include "renderer/tr_local.h"
 
+#include "Model_local.h"
+
+
 static const float CHECK_BOUNDS_EPSILON = 1.0f;
 
 /*
@@ -673,6 +676,8 @@ void R_LinkLightSurf( const drawSurf_t **link, const srfTriangles_t *tri, const
 	drawSurf->material = shader;
 	drawSurf->scissorRect = scissor;
 	drawSurf->dsFlags = 0;
+	drawSurf->particle_radius = 0.0f; // #3878
+
 	if ( viewInsideShadow ) {
 		drawSurf->dsFlags |= DSF_VIEW_INSIDE_SHADOW;
 	}
@@ -1186,7 +1191,8 @@ R_AddDrawSurf
 =================
 */
 void R_AddDrawSurf( const srfTriangles_t *tri, const viewEntity_t *space, const renderEntity_t *renderEntity,
-					const idMaterial *shader, const idScreenRect &scissor ) {
+					const idMaterial *shader, const idScreenRect &scissor, const float soft_particle_radius )
+{
 	drawSurf_t		*drawSurf;
 	const float		*shaderParms;
 	static float	refRegs[MAX_EXPRESSION_REGISTERS];	// don't put on stack, or VC++ will do a page touch
@@ -1198,7 +1204,17 @@ void R_AddDrawSurf( const srfTriangles_t *tri, const viewEntity_t *space, const
 	drawSurf->material = shader;
 	drawSurf->scissorRect = scissor;
 	drawSurf->sort = shader->GetSort() + tr.sortOffset;
-	drawSurf->dsFlags = 0;
+
+	if ( soft_particle_radius != -1.0f )	// #3878
+	{
+		drawSurf->dsFlags = DSF_SOFT_PARTICLE;
+		drawSurf->particle_radius = soft_particle_radius;
+	}
+	else
+	{
+		drawSurf->dsFlags = 0;
+		drawSurf->particle_radius = 0.0f;
+	}
 
 	// bumping this offset each time causes surfaces with equal sort orders to still
 	// deterministically draw in the order they are added
@@ -1217,7 +1233,8 @@ void R_AddDrawSurf( const srfTriangles_t *tri, const viewEntity_t *space, const
 			tr.viewDef->maxDrawSurfs *= 2;
 		}
 		tr.viewDef->drawSurfs = (drawSurf_t **)R_FrameAlloc( tr.viewDef->maxDrawSurfs * sizeof( tr.viewDef->drawSurfs[0] ) );
-		memcpy( tr.viewDef->drawSurfs, old, count );
+		if(count > 0)
+			memcpy( tr.viewDef->drawSurfs, old, count );
 	}
 	tr.viewDef->drawSurfs[tr.viewDef->numDrawSurfs] = drawSurf;
 	tr.viewDef->numDrawSurfs++;
@@ -1422,8 +1439,21 @@ static void R_AddAmbientDrawsurfs( viewEntity_t *vEntity ) {
 				vertexCache.Touch( tri->indexCache );
 			}
 
+			// Soft Particles -- SteveL #3878
+			float particle_radius = -1.0f;		// Default = disallow softening, but allow modelDepthHack if specified in the decl.
+			if ( r_useSoftParticles.GetBool() && r_enableDepthCapture.GetInteger() != 0
+				&& !shader->ReceivesLighting()          // don't soften surfaces that are meant to be solid
+				&& tr.viewDef->renderView.viewID >= 0 ) // Skip during "invisible" rendering passes (e.g. lightgem)
+			{
+				const idRenderModelPrt* prt = dynamic_cast<const idRenderModelPrt*>( def->parms.hModel ); // yuck.
+				if ( prt )
+				{
+					particle_radius = prt->SofteningRadius( surf->id );
+				}
+			}
+
 			// add the surface for drawing
-			R_AddDrawSurf( tri, vEntity, &vEntity->entityDef->parms, shader, vEntity->scissorRect );
+			R_AddDrawSurf( tri, vEntity, &vEntity->entityDef->parms, shader, vEntity->scissorRect, particle_radius );
 
 			// ambientViewCount is used to allow light interactions to be rejected
 			// if the ambient surface isn't visible at all
@@ -1513,6 +1543,14 @@ void R_AddModelSurfaces( void ) {
 			continue;
 		}
 
+		// Don't let particle entities re-instantiate their dynamic model during non-visible 
+		// views (in TDM, the light gem render) -- SteveL #3970
+		if ( tr.viewDef->renderView.viewID < 0
+			&& dynamic_cast<const idRenderModelPrt*>( vEntity->entityDef->parms.hModel ) != NULL ) // yuck.
+		{
+			continue;
+		}
+
 		// add the ambient surface if it has a visible rectangle
 		if ( !vEntity->scissorRect.IsEmpty() ) {
 			model = R_EntityDefDynamicModel( vEntity->entityDef );
