@@ -33,10 +33,8 @@ If you have questions concerning this license or the applicable additional terms
 #endif
 #define STB_VORBIS_NO_STDIO
 #define STB_VORBIS_NO_PUSHDATA_API // we're using the pulldata API
+#define STB_VORBIS_HEADER_ONLY
 #include "stb_vorbis.h"
-#undef L // the implementation part of stb_vorbis has these defines, they confuse other code..
-#undef C
-#undef R
 
 #include "sys/platform.h"
 #include "framework/FileSystem.h"
