@@ -83,7 +83,7 @@ toolTip_t DialogAFBody::toolTips[] = {
 	{ IDC_BUTTON_CM_BROWSE, "browse custom collision model" },
 	{ IDC_COMBO_BONE_JOINT1, "first joint of bone collision model" },
 	{ IDC_COMBO_BONE_JOINT2, "second joint of bone collision model" },
-	{ IDC_EDIT_CM_HEIGHT, "hieght of the collision model" },
+	{ IDC_EDIT_CM_HEIGHT, "height of the collision model" },
 	{ IDC_EDIT_CM_WIDTH, "width of the collision model" },
 	{ IDC_EDIT_CM_LENGTH, "length of the collision model" },
 	{ IDC_EDIT_CM_NUMSIDES, "number of sides of the collision model" },
@@ -707,7 +707,7 @@ BOOL DialogAFBody::OnInitDialog()  {
 DialogAFBody::OnToolHitTest
 ================
 */
-int DialogAFBody::OnToolHitTest( CPoint point, TOOLINFO* pTI ) const {
+INT_PTR DialogAFBody::OnToolHitTest( CPoint point, TOOLINFO* pTI ) const {
 	CDialog::OnToolHitTest( point, pTI );
 	return DefaultOnToolHitTest( toolTips, this, point, pTI );
 }
