@@ -57,7 +57,7 @@ void DisplayRealTimeString( const char *string, ... ) {
 		va_start( argPtr, string );
 		vsprintf( buf, string, argPtr );
 		va_end( argPtr );
-		common->Printf( buf );
+		common->Printf( "%s", buf );
 		lastUpdateTime = time;
 	}
 }
