@@ -26,10 +26,9 @@ If you have questions concerning this license or the applicable additional terms
 ===========================================================================
 */
 
-#include "../../idlib/precompiled.h"
-#pragma hdrstop
+#include "tools/edit_gui_common.h"
+
 
-#include "../../game/game.h"
 #include "../../sys/win32/win_local.h"
 #include "../../sys/win32/rc/common_resource.h"
 #include "../../sys/win32/rc/Radiant_resource.h"
@@ -1273,7 +1272,16 @@ BOOL CDialogParticleEditor::OnInitDialog() {
 
 void CDialogParticleEditor::OnHScroll( UINT nSBCode, UINT nPos, CScrollBar* pScrollBar ) {
 	CDialog::OnHScroll( nSBCode, nPos, pScrollBar );
-	CSliderCtrl *ctrl = dynamic_cast< CSliderCtrl* >( pScrollBar );
+
+	// DG: from SteelStorm2:
+	// Something funky is going on with the RTTI.  The dynamic_cast even to a CRangeSlider*
+	// was not happening correctly.  Whatever is getting to this callback from the CRangeSlider
+	// must not be one of the basic slider types or even a CRangeSlider.  What is weird is the
+	// objects coming in have the same addresses as the various CRangeSliders on the dlg, so they
+	// should cast correctly.  Turns out it does not matter because once the addresses matche up
+	// this code just uses the dlg member reference so all I need is for the address check to
+	// go through correctly.
+	CRangeSlider* ctrl = (CRangeSlider*)pScrollBar;
 	if ( !ctrl ) {
 		return;
 	}
